use bevy::prelude::*;

#[derive(Resource, Default, Debug, Clone, PartialEq)]
pub struct Input {
    pub movement: Vec2,
    pub angle: f32,
    pub action: bool,
    pub cancel: bool,
}

pub fn process_input(
    mut input: ResMut<Input>,
    keys: Res<ButtonInput<KeyCode>>,
    // mouse: Res<ButtonInput<MouseButton>>,
) {
    input.movement.x = 0.;
    input.movement.y = 0.;
    for key in keys.get_pressed() {
        match key {
            KeyCode::KeyA | KeyCode::ArrowLeft => {
                input.movement -= Vec2::X;
            }
            KeyCode::KeyD | KeyCode::ArrowRight => {
                input.movement += Vec2::X;
            }
            KeyCode::KeyW | KeyCode::ArrowUp => {
                input.movement += Vec2::Y;
            }
            KeyCode::KeyS | KeyCode::ArrowDown => {
                input.movement -= Vec2::Y;
            }
            _ => {}
        }
    }
    input.movement = input.movement.normalize_or_zero();
    // input.target // not set using keyboard, see mouse
    input.action = input.action || keys.any_pressed([KeyCode::Space, KeyCode::Enter]);
    input.cancel = input.cancel || keys.just_pressed(KeyCode::Escape);

    info!("Input: {:#?}", input);
}

// TODO: controller input: https://bevy-cheatbook.github.io/input/gamepad.html
// Iterate controllers on startup and assign to players/units?
// What logic for handling controller connection/disconnection?
// Will we have splitscreen?
// Once a controller is assigned to a player, we can handle its input
// pub fn controller_input(
//     axes: Res<Axis<GamepadAxis>>,
//     buttons: Res<ButtonInput<GamepadButton>>,
//     my_gamepad: Option<Res<MyGamepad>>,
// ) {
// }
