use crate::camera::Watched;
use crate::movement::{Heading, Mover, Speed, Velocity, YSortable};
use crate::prelude::*;
use bevy::sprite::MaterialMesh2dBundle;

const PLAYER_SPEED: f32 = 1000.;

// #[derive(Resource, Default)]
// pub struct Layout(Handle<TextureAtlasLayout>);

// #[derive(Resource)]
// pub struct CrosshairMaterial(Handle<ColorMaterial>);

// #[derive(Resource)]
// pub struct CrosshairMesh(Mesh2dHandle);

#[derive(Component, Debug)]
pub struct Player;

#[derive(Component, Debug)]
pub struct Crosshair;

pub fn startup(
    mut commands: Commands,
    assets: Res<AssetServer>,
    mut meshes: ResMut<Assets<Mesh>>,
    mut materials: ResMut<Assets<ColorMaterial>>,
) {
    let layout = assets.add(TextureAtlasLayout::from_grid(
        UVec2::new(32, 64),
        3,
        1,
        Some(UVec2 { x: 0, y: 0 }),
        Some(UVec2 { x: 0, y: 0 }),
    ));
    commands
        .spawn((
            Player,
            YSortable,
            Watched,
            SpriteBundle {
                texture: assets.load("img/Player.png"),
                ..default()
            },
            TextureAtlas {
                layout,
                ..default()
            },
            Mover {
                velocity: Velocity(Vec2::ZERO),
                heading: Heading(Vec2::ZERO),
                speed: Speed(PLAYER_SPEED),
            },
        ))
        .with_children(|player| {
            let mesh = meshes.add(Capsule2d::new(3.0, 25.0));
            let material = materials.add(Color::hsl(360. * 1 as f32 / 3 as f32, 0.95, 0.7));
            // let global_transform = GlobalTransform::from_xyz(
            //     0.0, 20.0,
            //     // TODO: need some way to ensure this draws above everything else?
            //     // a UI layer or something?
            //     1000.,
            // );

            let transform = Transform::from_xyz(
                0.0, 0.0,
                // TODO: need some way to ensure this draws above everything else?
                // a UI layer or something?
                1000.,
            );

            player.spawn((
                Crosshair,
                // TODO: crosshair transform?
                MaterialMesh2dBundle {
                    // global_transform,
                    mesh: mesh.into(),
                    material,
                    transform,
                    ..default()
                },
            ));
            player.spawn((SpatialBundle::default(), SpatialListener::new(1.)));
            player.spawn((Text2dBundle {
                text: Text::from_section(
                    "You",
                    TextStyle {
                        font_size: 18.,
                        color: Color::WHITE,
                        ..default()
                    },
                ),
                transform: Transform::from_translation(Vec3::new(30.0, -30.0, 0.)),
                ..default()
            },));
        });
}

pub fn control(input: Res<Input>, mut player: Query<&mut Heading, With<Player>>) {
    if let Ok(mut heading) = player.get_single_mut() {
        heading.0 = input.movement;
    }
}

pub fn exit(mut commands: Commands, q: Query<Entity, With<Player>>) {
    for id in q.iter() {
        commands.entity(id).despawn_recursive();
    }
}
